from ppms import ppms
from . import messages as msg
from . import info_handler

def get_system_cal_id():
    # system variable @3 contains the license id 
    # dg345->di026510 contains the base calendar id
    # with that one we can find the base calendar record in dg418
    license = ppms.uvar_get('@3')
    if not license:
        info_handler.warn(msg.get_const('missing_license'))
        return
    system_record = ppms.search_record(345, [license], [26510], True)
    if not system_record:
        info_handler.warn(msg.get_const('syscal_fetch_failed'))
        return
    return system_record.get_di_by_id(26510).get_value()

def get_calendar_rec(cal_id=None):
    if cal_id is None:
        cal_id = get_system_cal_id()
        if cal_id is None:
            info_handler.warn(msg.get_const('no_calid'))
            return

    base_cal_rec = ppms.search_record(418, [cal_id], [305, 476, 477, 1563, \
        59221, 59222, 59223, 59224, 59225, 59226, 59227], True)
    return base_cal_rec

def get_periods(calendar_rec):
    return calendar_rec.get_children(430)

# Searches the project record by id. TODO: do we need the dilist?
def get_proj_rec(project_id):
    di_list = [1001, 7802, 1005, 1006, 1016, 1023, 1024, 690, 1025, 1026]
    proj_rec = ppms.search_record(461, [project_id], di_list, True)
    return proj_rec

# returns all tasks of the record
def get_task_records(proj_rec):
    return proj_rec.get_children(463)

# returns a tuple of the relation records between task and resource and the resources
def get_resources_for_task(task_rec):
    resource_tuples = list()

    for link in task_rec.get_children(466):
        resources = link.get_children(467)
        if len(resources) > 0:
            resource_tuples.append((link, resources[0]))
        else:
            info_handler.warn(msg.get_const('invalid_resource'))
    return resource_tuples
    
# returns the links between tasks
def get_links_for_task(task_rec):
    return task_rec.get_children(465)

def get_resource_periods(res_rec):
    return res_rec.get_children(468)
