from ..format import *
from ..mspdi_2003_classes import *

def transform(planta_record, destination_object, transformation_triple):
    (di_id, format_func, setter_func) = transformation_triple
    dtp_di = planta_record.get_di_by_id(di_id)
    if not dtp_di:
        #TODO customized Exception. alternatively: log file as mash proposed
        return
    pp_value = dtp_di.get_value()
    msp_value = format_func(pp_value)
    if not msp_value:
        #TODO customized Exception. alternatively: log file as mash proposed
        return
    setter_func(destination_object, msp_value)

def transform_whole_node(planta_record, destination_object, transformation_triples):
    for triple in transformation_triples:
        transform(planta_record, destination_object, triple)
    
# The following transformation_functions will only transform data in one record
# that can be mapped to the msp equivalent without a larger context
# All transformation that relies on knowing overall structure/a bigger context
# has to be implemented in the outer recursive transformation construct
def transform_project_data(proj_record, proj_object):
    project_triples = [
        # (DI-ID,    format_func,    setter_func)
        (7802, identity, Project.set_Company),
        (1005, start_date_to_msp_date, Project.set_CreationDate),
        (1006, finish_date_to_msp_date, Project.set_LastSaved),
        (1024, finish_date_to_msp_date, Project.set_FinishDate),
        (690, identity, Project.set_Name),
        (41317, identity, Project.set_Title)
    ]
    
    transform_whole_node(proj_record, proj_object, project_triples)
    
def transform_calendar_data(cal_record, cal_object):
    cal_triples = [
        # (DI-ID,    format_func,    setter_func)
        (305, identity, Calendar.set_Name)
    ]
    
    transform_whole_node(cal_record, cal_object, cal_triples)

def transform_period_data(period_record, period_object):
    period_triples = [
        # (DI-ID,    format_func,    setter_func)
    ]
    transform_whole_node(period_record, period_object, period_triples)
    
    
    
def transform_task_data(task_record, task_object):
    task_triples = [
        # (DI-ID,    format_func,    setter_func)
        (1099, start_date_to_msp_date, Task.set_CreateDate),
        (807, identity, Task.set_Name),
        (1119, start_date_to_msp_date, Task.set_EarlyStart),
        (1120, finish_date_to_msp_date, Task.set_EarlyFinish),
        (1119, start_date_to_msp_date, Task.set_LateStart),
        (1120, finish_date_to_msp_date, Task.set_LateFinish),
        (1119, start_date_to_msp_date, Task.set_Start),
        (1120, finish_date_to_msp_date, Task.set_Finish),
        (1119, start_date_to_msp_date, Task.set_ConstraintDate),
        (1121, start_date_to_msp_date, Task.set_ActualStart),
        (1122, finish_date_to_msp_date, Task.set_ActualFinish),
        (1132, days_to_msp_hours, Task.set_ActualDuration),
        (1133, days_to_msp_hours, Task.set_RemainingDuration),
        (24115, identity, Task.set_Milestone),
        (8845, identity, Task.set_Summary)
    ]
    
    transform_whole_node(task_record, task_object, task_triples)

def transform_assignment_data(assgmnt_record, assgmnt_object):
    assgmnt_triples = [
        # (DI-ID,    format_func,    setter_func)
        #(1410, hours_to_msp_hours, Assignment.set_Work),
        #(1416, hours_to_msp_hours, Assignment.set_ActualWork),
        #(1423, hours_to_msp_hours, Assignment.set_RemainingWork),
        (1420, start_date_to_msp_date, Assignment.set_ActualStart),
        (1422, finish_date_to_msp_date, Assignment.set_ActualFinish),
        (1441, start_date_to_msp_date, Assignment.set_Start),
        (1442, finish_date_to_msp_date, Assignment.set_Finish)
    ]
    
    transform_whole_node(assgmnt_record, assgmnt_object, assgmnt_triples)

def transform_resource_data(res_record, res_object):
    res_triples = [
        # (DI-ID,    format_func,    setter_func)
        (1218, identity, Resource.set_Name),
        (1281, identity, Resource.set_Group),
        #(24112, identity, Resource.set_Type),
        (24114, identity, Resource.set_MaxUnits),
        (1232, identity, Resource.set_MaterialLabel)
    ]
    
    transform_whole_node(res_record, res_object, res_triples)

def transform_link_data(link_record, link_object):
    link_triples = [
        # (DI-ID,    format_func,    setter_func)
        (1206, link_type_to_msp_link_type, PredecessorLink.set_Type)
    ]
    
    transform_whole_node(link_record, link_object, link_triples)
