from . import transform_data
from . import default_values
from . import transform_calendar

from ..mspdi_2003_classes import *
from .. import format
from .. import fetch_project_data as fetch_data
from .. import info_handler
from .. import messages as msg

from ppms import ppms

def transform_resource(resource_rec, res_uid):
    resource_obj = Resource(UID=res_uid, ID=res_uid)
    transform_data.transform_resource_data(resource_rec, resource_obj)
    return resource_obj

def transform_assignment(assignment_rec, task_uid, res_uid):
    assignment_obj = Assignment(TaskUID=task_uid, ResourceUID=res_uid)
    transform_data.transform_assignment_data(assignment_rec, assignment_obj)
    return assignment_obj

def resource_index(res_list, res_rec):
    index = 0
    for element in res_list:
        if element.res_id.get_value() == res_rec.res_id.get_value():
            return index
        index += 1
    return -1

def _set_effort(assignment_rec, assignment_obj, resource_rec):
    # hour unit is default for work
    format_func = format.hours_to_msp_hours
    
    if resource_rec.basic_unit.get_value() == 'T':
        format_func = format.days_to_msp_hours
    
    assignment_obj.set_Work(format_func(
        assignment_rec.effort_plan.get_value()))
    assignment_obj.set_ActualWork(format_func(
        assignment_rec.effort_act.get_value()))
    assignment_obj.set_RemainingWork(format_func(
        assignment_rec.effort_rem.get_value()))

def res_type(resource_rec):
    resource_unit = resource_rec.basic_unit.get_value()
    if resource_unit in ['h', 'T']:
        return 1
    elif resource_unit in ['KG', 'Stck']:
        return 0
    else:
        return -1

        
def process_resource_tuples(tasks, task_objs, project_obj):
    resources = list()
    resource_objs = list()   
    assignments_obj = Assignments()
    resources_obj = Resources()
    
    for task_rec in tasks:
        task_uid = tasks.index(task_rec) + 1
        latest_end_date = 0
        for assignment_rec, resource_rec in \
            fetch_data.get_resources_for_task(task_rec):
            
            #add resource to list, if it is not in there already
            res_index = resource_index(resources, resource_rec)
            if res_index == -1:
                info_handler.info(msg.get_const('resource_transformation').format(
                    resource_rec.res_id.get_value()))
                if res_type(resource_rec) == -1:
                    # we do not process resources and their assignments if
                    # they are cost or budget resources for now
                    info_handler.warn(msg.get_const('resource_type_invalid').format(
                        resource_rec.res_id.get_value(),
                        resource_rec.basic_unit.get_value()))
                    continue

                resources.append(resource_rec)
                res_index = len(resources) - 1
                resource_objs.append(transform_resource(
                    resources[res_index], res_index + 1))
                resource_objs[-1].set_Type(res_type(resource_rec))
                resource_objs[-1].set_CalendarUID(
                    transform_calendar.build_resource_cal(
                    resource_rec, project_obj))

            res_uid = res_index + 1
            res_obj = resource_objs[res_index]
            #process assignments
            
            # needed to set the Task Type to fixed_duration in the case a task
            # has a duration set that exceeds the computed duration
            if assignment_rec.effort_plan.get_value() > 0.0:
                if assignment_rec.calc_end.get_value() > latest_end_date:
                    latest_end_date = assignment_rec.calc_end.get_value()
            
            info_handler.info(msg.get_const('assignment_transformation').format(resource_rec.res_id.get_value(), 
                task_rec.task_id.get_value()))
            
            assignment_obj = transform_assignment(
                assignment_rec, task_uid, res_uid)
             
            if res_obj.get_Type() == 0: #Material
                assignment_obj.set_Units(assignment_rec.effort_plan.get_value())
            else:
                _set_effort(assignment_rec, assignment_obj, resource_rec)
                res_obj.set_MaterialLabel(None)
                
            assignments_obj.add_Assignment(assignment_obj)    
            
        if latest_end_date < task_rec.calc_end.get_value():
            task_objs[task_uid - 1].set_Type(1)
    
    project_obj.set_Assignments(assignments_obj)
    
    resources_obj.set_Resource(resource_objs)
    project_obj.set_Resources(resources_obj)
    
