from . import transform_data
from . import transform_task_structure
from . import task_reimport

from .. import mspdi_2003_classes
from .. import mspdi_extended_fields
from .. import fetch_project_data
from .. import info_handler
from .. import messages as msg
from .. import task_hash

from ppms.ppms_cu import Helper
from ppms.ppms_cu import Access
from ppms.file_transfer import get_file_from_client
from ppms import ppms

def get_project_file(project_id):
    get_file_from_client.send_get_file_task('MSP Import',
        'XML Files (*.xml)|*.xml|All files (*.*)|*.*', True,
        '''from ppms.msp_interface.msp_import import main_import
main_import.import_project("''' + project_id + '", "{0}")')

def reload_schedule_mod(proj_rec):
    from ppms.module_subclasses.schedule_module import ScheduleModule
    
    schedule_mod = ppms.get_focused_module()
    if schedule_mod and type(schedule_mod) == type(ScheduleModule(None)):
        schedule_mod.menu(12) #search the new task records
        schedule_mod.open_module('0099TT') #scheduling step
        info_handler.info(msg.get_const('scheduled'))
    else:
        info_handler.warn(msg.get_const('scheduling_failed'))

def start_comparison_mod(proj_rec):
    proj_id = proj_rec.pr_id.get_value()
    settings = Helper.get_global_setting('msp_reimport_comparison')
    if settings:
        module_id = settings.alpha120.get_value()
        if module_id != '':
            schedule_mod = ppms.get_focused_module()
            schedule_mod.set_new_L_var(74, [proj_id])
            access = Access(schedule_mod)
            if module_id in access.get_submodules():
                reimport_mod = access.get_submodules()[module_id]
                reimport_mod.menu(19)
            else:
                reimport_mod = schedule_mod.open_module(module_id, skip_macro=False,
                    forced_status = 2, dock_to_module=schedule_mod.get_uid(),
                    dock_style=0, focus=0)
            reimport_mod.focus()
            reimport_mod.menu(12)
    info_handler.refocus()

def edited_by_msp(project):
    return project.get_CurrentDate() != None

def import_project(project_id, import_filename):
    #prevent summing up when exporting multiple times in one session
    info_handler.init_summary()
    
    info_handler.start_ui()

    memfile = get_file_from_client.get_file(import_filename) 
    if memfile:
        try:
            project_instance = mspdi_2003_classes.parse(memfile)
            info_handler.info(msg.get_const('import_transfer'))
        except Exception as e:
            info_handler.error(msg.get_const('import_parsing_failed').format(str(e)))
            info_handler.finish_ui()
            return
    else:
        info_handler.error(msg.get_const('import_transfer_failed'))
        info_handler.finish_ui()
        return

    try:
        # check for PLANTA ID Field
        generated_by_planta = False
        if mspdi_extended_fields.has_planta_field_cust(project_instance):
            generated_by_planta = True
            info_handler.info(msg.get_const('planta_reimport'))

        task_hash.run_link_switch(project_instance.get_Tasks().get_Task())

        # get project record
        proj_rec = fetch_project_data.get_proj_rec(project_id)

        if not generated_by_planta:
            transform_data.transform_project_data(proj_rec, project_instance)
            proj_rec.save()
            info_handler.info(msg.get_const('import_project').format(project_id))
            #check model paramters
            fix_tasks = False
            model = Helper.get_active_license_model()
            if model is not None:
                if not model.ta_rsd_red.get_value():
                    info_handler.warn(msg.get_const('no_requested_dates'))
                if model.use_todays_date.get_value():
                    info_handler.info(msg.get_const('fixing_tasks'))
                    fix_tasks = True
            #task transformation
            transform_task_structure.transform_tasks(project_instance, proj_rec, fix_tasks)
            reload_schedule_mod(proj_rec)
        else:
            #reimport the project (w.r.t. tasks and links)
            if not edited_by_msp(project_instance):
                info_handler.warn(msg.get_const('no_msp_edit'))
            task_reimport.reimport_tasks(project_instance, proj_rec)
            #Here the reimport module will be started
            start_comparison_mod(proj_rec)
    except Exception as e:
        info_handler.error(msg.get_const('exception_exit').format(e))
        info_handler.finish_ui()
        return

    info_handler.finish_ui()
