from .mspdi_2003_classes import Task
from .mspdi_2003_classes import PredecessorLink as Link
from . import info_handler
from . import mspdi_extended_fields
from .msp_export import transform_data
from hashlib import md5

def get_link_identifiers(task):
    link_identifiers = []
    if hasattr(task, 'succ_links'):
        links = sorted(task.succ_links, key=lambda task: task[0])
        for link in links:
            link_identifiers.append('{0}_{1}'.format(link[0], link[1]))
    identifier_str = '_'.join(link_identifiers)
    return identifier_str

task_relevant_attributes = [
        Task.get_Name,
        get_link_identifiers,
        Task.get_Start,
        Task.get_Finish,
        Task.get_EarlyStart,
        Task.get_EarlyFinish,
        Task.get_LateStart,
        Task.get_LateFinish,
        Task.get_Duration
    ]

def calc_hash(obj, attr_getter_list):
    attr_str = ''
    for func in attr_getter_list:
        attr = func(obj)
        if attr is not None:
            attr_str += str(attr)
    try:
        enc_str = attr_str.encode()
        return md5(enc_str).hexdigest()
    except BaseException as e:
        info_handler.info('Encoding error occured: ' + str(e))
        return ''

def calc_task_hash(task_obj):
    relevant_attrs = task_relevant_attributes
    if task_obj.get_Summary() in ['1', 1, 'true', True]:
        relevant_attrs = task_relevant_attributes[0:2]
    elif task_obj.get_Milestone() in ['1', 1, 'true', True]:
        relevant_attrs = task_relevant_attributes[0:3]
    hash_str = calc_hash(task_obj, relevant_attrs)
    return hash_str

def set_task_hash(task_obj):
    mspdi_extended_fields.add_extended_field(task_obj,
        ('TaskHash', calc_task_hash(task_obj)))

def get_task_hash(task_obj):
    mspdi_extended_fields.get_extended_field_value(
        task_obj, 'TaskHash')

# this is just for having the link ids under the predecessor record,
# just as it is in PLANTA (otherwise, link changes would not be visible
# on the "right" record
def run_link_switch(tasks):
    task_id_dict = dict([(task.get_UID(),task) for task in tasks])
    for id in task_id_dict:
        for link in task_id_dict[id].get_PredecessorLink():
            predecessor = task_id_dict[link.get_PredecessorUID()]
            if hasattr(predecessor, 'succ_links'):
                predecessor.succ_links.append((id, link.get_Type()))
            else:
                predecessor.succ_links = [((id, link.get_Type()))]