# -*- coding: utf-8 -*-
"""Functions for working with dialogs"""
from ppms import ppms
from ppms import text_constant
from ppms.constants import *


__all__ = ['ButtonList', 'ask_user_for_confirmation', 'ask_for_confirmation_via_dialog']


class Button(object):
    YES = '000030'
    NO = '000031'

    OK = '000032'
    BACK = '000033'


class ButtonList(object):
    YES_NO = 1
    OK_BACK = 2

    @classmethod
    def get_button_text_from_value(cls, value):
        if value == cls.YES_NO:
            return [text_constant.get_text_constant(Button.YES, default='Yes'),
                    text_constant.get_text_constant(Button.NO, default='No')]

        elif value == cls.OK_BACK:
            return [text_constant.get_text_constant(Button.OK, default='Ok'),
                    text_constant.get_text_constant(Button.BACK, default='Back')]

        return None


def ask_user_for_confirmation(message_text, button_enum, ole_id=OLE_QUESTION_MARK):
    """Display a message and check if the reply is MSG_REPLY_YES

    message_text: The text that should be displayed
    button_enum: One of the ButtonList enum values
    """

    # We need to fetch the text constants for the buttons at runtime so they
    # are in the correct language for the current user
    buttons = ButtonList.get_button_text_from_value(value=button_enum)
    if buttons is None:
        raise ValueError('Invalid button_enum value: {}'.format(button_enum))

    ppms.ui_message_box(message_text, ole_id=ole_id, button_list=buttons)

    return ppms.msg_pop().get_reply() == MSG_REPLY_YES


def ask_for_confirmation_via_dialog(dialog_message_id):
    """Display a dialog message and check if the reply is MSG_REPLY_YES"""
    ppms.ui_message_id(dialog_message_id)

    return ppms.msg_pop().get_reply() == MSG_REPLY_YES